@@ -60,7 +60,15 @@ static CairoOutFontSubst cairoOutSubstFonts[16] = {
 // CairoFont
 //------------------------------------------------------------------------
 
-CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
+/* For the 0.3 release of poppler we're creating a CairoFont, and
+ * thus, an FT_Face for each size of each font we encounter.  We
+ * should of course be able to share FT_Face instances across all
+ * sizes of a font.  However, to do that we need to be able to create
+ * a cairo_unscaled_font_t, which is not possible with the cairo-0.4.0
+ * API which is what we're targeting with this release. */
+
+CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib, 
+		     double m11, double m12, double m21, double m22) {
   Ref embRef;
   Object refObj, strObj;
   GooString *tmpFileName, *fileName, *substName,*tmpFileName2;
@@ -72,6 +80,7 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
   char *name;
   FoFiTrueType *ff;
   FoFiType1C *ff1c;
+  cairo_matrix_t *matrix;
   
   codeToGID = NULL;
   codeToGIDLen = 0;
@@ -82,7 +91,7 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
   fontType = gfxFont->getType();
 
   tmpFileName = NULL;
-  
+
   if (gfxFont->getEmbeddedFontID(&embRef)) {
     if (!openTempFile(&tmpFileName, &tmpFile, "wb", NULL)) {
       error(-1, "Couldn't create temporary font file");
@@ -156,6 +165,7 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
   case fontType1:
   case fontType1C:
     if (FT_New_Face(lib, fileName->getCString(), 0, &face)) {
+      error(-1, "could not create type1 face");
       goto err2;
     }
     
@@ -180,6 +190,7 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
     /* Fall through */
   case fontTrueType:
     if (!(ff = FoFiTrueType::load(fileName->getCString()))) {
+      error(-1, "failed to load truetype font\n");
       goto err2;
     }
     /* This might be set already for the CIDType2 case */
@@ -189,6 +200,7 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
     }
     if (!openTempFile(&tmpFileName2, &tmpFile, "wb", NULL)) {
       delete ff;
+      error(-1, "failed to open truetype tempfile\n");
       goto err2;
     }
     ff->writeTTF(&fileWrite, tmpFile);
@@ -196,6 +208,7 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
     delete ff;
 
     if (FT_New_Face(lib, tmpFileName2->getCString(), 0, &face)) {
+      error(-1, "could not create truetype face\n");
       goto err2;
     }
     unlink (tmpFileName2->getCString());
@@ -212,8 +225,9 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
       codeToGID = NULL;
       codeToGIDLen = 0;
     }
-    if (FT_New_Face(lib, tmpFileName2->getCString(), 0, &face)) {
+    if (FT_New_Face(lib, tmpFileName->getCString(), 0, &face)) {
       gfree(codeToGID);
+      error(-1, "could not create cid face\n");
       goto err2;
     }
     break;
@@ -231,6 +245,21 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
     unlink (fileName->getCString());
   }
 
+  this->m11 = m11;
+  this->m12 = m12;
+  this->m21 = m21;
+  this->m22 = m22;
+  matrix = cairo_matrix_create ();
+  cairo_matrix_set_affine (matrix, m11, m12, m21, m22, 0, 0);
+  cairo_font = cairo_ft_font_create_for_ft_face (face, FT_LOAD_NO_HINTING,
+						 matrix);
+  cairo_matrix_destroy (matrix);
+  if (cairo_font == NULL) {
+    error(-1, "could not create cairo font\n");
+    goto err2; /* this doesn't do anything, but it looks like we're
+		* handling the error */
+  }
+
   return;
  err2:
   /* hmm? */
@@ -238,21 +267,27 @@ CairoFont::CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib) {
 }
 
 CairoFont::~CairoFont() {
-  /* TODO: Free the face!
-   * How do we know when we can do this?
-   * It might be referenced by a cairo cache
-   */
+  cairo_font_destroy (cairo_font);
+
+  /* cairo_font_t's created from an FT_Face are never cached so we can
+   * free the font here.  There might be glyphs in the cairo glyph
+   * cache referencing this font, but since we're throwing this font
+   * away, they won't be used and will slowly fall out of the
+   * cache. */
+  FT_Done_Face (face);
 }
 
 GBool
-CairoFont::matches(Ref &other) {
-  return (other.num == ref.num &&
-	  other.gen == ref.gen);
+CairoFont::matches(Ref &other,
+		   double m11, double m12, double m21, double m22) {
+  return (other.num == ref.num && other.gen == ref.gen &&
+	  this->m11 == m11 && this->m12 == m12 &&
+	  this->m21 == m21 && this->m22 == m22);
 }
 
 cairo_font_t *
-CairoFont::getFont(cairo_matrix_t *font_scale) {
-    return cairo_ft_font_create_for_ft_face (face, FT_LOAD_DEFAULT, font_scale);
+CairoFont::getFont(void) {
+  return cairo_font;
 }
 
 unsigned long
@@ -324,7 +359,8 @@ CairoFontEngine::~CairoFontEngine() {
 }
 
 CairoFont *
-CairoFontEngine::getFont(GfxFont *gfxFont, XRef *xref) {
+CairoFontEngine::getFont(GfxFont *gfxFont, XRef *xref,
+			 double m11, double m12, double m21, double m22) {
   int i, j;
   Ref ref;
   CairoFont *font;
@@ -332,20 +368,15 @@ CairoFontEngine::getFont(GfxFont *gfxFont, XRef *xref) {
   
   fontType = gfxFont->getType();
   if (fontType == fontType3) {
-    printf ("Type 3 font!\n");
     /* Need to figure this out later */
     //    return NULL;
   }
 
   ref = *gfxFont->getID();
 
-  font = fontCache[0];
-  if (font && font->matches(ref)) {
-    return font;
-  }
-  for (i = 1; i < cairoFontCacheSize; ++i) {
+  for (i = 0; i < cairoFontCacheSize; ++i) {
     font = fontCache[i];
-    if (font && font->matches(ref)) {
+    if (font && font->matches(ref, m11, m12, m21, m22)) {
       for (j = i; j > 0; --j) {
 	fontCache[j] = fontCache[j-1];
       }
@@ -354,7 +385,7 @@ CairoFontEngine::getFont(GfxFont *gfxFont, XRef *xref) {
     }
   }
   
-  font = new CairoFont (gfxFont, xref, lib);
+  font = new CairoFont (gfxFont, xref, lib, m11, m12, m21, m22);
   if (fontCache[cairoFontCacheSize - 1]) {
     delete fontCache[cairoFontCacheSize - 1];
   }
