@@ -198,6 +198,7 @@ void CairoOutputDev::updateStrokeColor(GfxState *state) {
 
 void CairoOutputDev::updateFont(GfxState *state) {
   cairo_font_t *font;
+  double m11, m12, m21, m22;
   double w;
 
   LOG(printf ("updateFont() font=%s\n", state->getFont()->getName()->getCString()));
@@ -205,28 +206,20 @@ void CairoOutputDev::updateFont(GfxState *state) {
   /* Needs to be rethough, since fonts are now handled by cairo */
   needFontUpdate = gFalse;
 
-  currentFont = fontEngine->getFont (state->getFont(), xref);
-
-  double m11, m12, m21, m22;
-  
   state->getFontTransMat(&m11, &m12, &m21, &m22);
   m11 *= state->getHorizScaling();
   m12 *= state->getHorizScaling();
 
-  LOG(printf ("font matrix: %f %f %f %f\n", m11, m12, m21, m22));
+  /* w = currentFont->getSubstitutionCorrection(state->getFont()); */
+  m12 *= -1;
+  m22 *= -1;
 
-  w = currentFont->getSubstitutionCorrection(state->getFont());
+  LOG(printf ("font matrix: %f %f %f %f\n", m11, m12, m21, m22));
   
-  cairo_matrix_t *matrix = cairo_matrix_create ();
-  cairo_matrix_set_affine (matrix,
-			   m11, m21,
-			   -m12*w, -m22*w,
-			   0, 0);
-  
-  font = currentFont->getFont(matrix);
-  if (font)
-    cairo_set_font (cairo, font);
-  cairo_matrix_destroy (matrix);
+  currentFont = fontEngine->getFont (state->getFont(), xref,
+				     m11, m21, m12, m22);
+  font = currentFont->getFont();
+  cairo_set_font (cairo, font);
 }
 
 void CairoOutputDev::doPath(GfxState *state, GfxPath *path,
@@ -463,38 +456,36 @@ void CairoOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   cairo_surface_t *image;
   int x, y;
   ImageStream *imgStr;
-  Guchar pix;
+  Guchar *pix;
   double *ctm;
   cairo_matrix_t *mat;
+  int invert_bit;
 
-  /* TODO: I'm not sure how to implement this, and its currently crashing.
-     Disabling for now */
-
-  return;
-  
-  buffer = (char *)malloc (width * height);
-
+  buffer = (char *)malloc (width * height * 4);
   if (buffer == NULL) {
     error(-1, "Unable to allocate memory for image.");
     return;
   }
 
   /* TODO: Do we want to cache these? */
-  imgStr = new ImageStream(str, width,
-			   1, 1);
+  imgStr = new ImageStream(str, width, 1, 1);
   imgStr->reset();
 
+  invert_bit = invert ? 1 : 0;
+
   for (y = 0; y < height; y++) {
-    dest = buffer + y * width;
+    pix = imgStr->getLine();
+    dest = buffer + y * width * 4;
     for (x = 0; x < width; x++) {
-      imgStr->getPixel(&pix);
-      if (invert)
-	pix ^= 1;
-      
-      if (pix)
-	*dest++ = 255;
-      else
+
+      *dest++ = soutRound(255 * fill_color.b);
+      *dest++ = soutRound(255 * fill_color.g);
+      *dest++ = soutRound(255 * fill_color.r);
+ 
+      if (pix[x] ^ invert_bit)
 	*dest++ = 0;
+      else
+	*dest++ = 255;
     }
   }
 
@@ -514,14 +505,9 @@ void CairoOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   cairo_concat_matrix (cairo, mat);
   cairo_matrix_destroy (mat);
 
-  /* TODO: Should we use A1 here? I assume that is bit-packed */
   image = cairo_surface_create_for_image (
-              buffer, CAIRO_FORMAT_A8, width, height, width);
+              buffer, CAIRO_FORMAT_ARGB32, width, height, width * 4);
   cairo_surface_set_filter (image, CAIRO_FILTER_BEST);
-
-  /* TODO: Is this the right way to do image masks? */
-  cairo_set_rgb_color (cairo,
-		       fill_color.r, fill_color.g, fill_color.b);
   cairo_show_surface (cairo, image, width, height);
 
   cairo_restore (cairo);
