@@ -13,6 +13,7 @@
 #endif
 
 #include <stddef.h>
+#include <stdlib.h>
 #include "goo/gmem.h"
 #include "Object.h"
 #include "XRef.h"
@@ -21,6 +22,7 @@
 #include "Page.h"
 #include "Error.h"
 #include "Link.h"
+#include "PageLabelInfo.h"
 #include "Catalog.h"
 
 //------------------------------------------------------------------------
@@ -39,6 +41,7 @@ Catalog::Catalog(XRef *xrefA) {
   pageRefs = NULL;
   numPages = pagesSize = 0;
   baseURI = NULL;
+  pageLabelInfo = NULL;
 
   xref->getCatalog(&catDict);
   if (!catDict.isDict()) {
@@ -90,10 +93,48 @@ Catalog::Catalog(XRef *xrefA) {
   catDict.dictLookup("Dests", &dests);
 
   // read root of named destination tree
-  if (catDict.dictLookup("Names", &obj)->isDict())
-    obj.dictLookup("Dests", &nameTree);
-  else
-    nameTree.initNull();
+  if (catDict.dictLookup("Names", &obj)->isDict()) {
+    obj.dictLookup("Dests", &obj2);
+    destNameTree.init(xref, &obj2);
+    obj2.free();
+  }
+  obj.free();
+
+  if (catDict.dictLookup("PageLabels", &obj)->isDict())
+    pageLabelInfo = new PageLabelInfo(&obj, numPages);
+  obj.free();
+
+  // read page mode
+  pageMode = pageModeNone;
+  if (catDict.dictLookup("PageMode", &obj)->isName()) {
+    if (obj.isName("UseNone"))
+      pageMode = pageModeNone;
+    else if (obj.isName("UseOutlines"))
+      pageMode = pageModeOutlines;
+    else if (obj.isName("UseThumbs"))
+      pageMode = pageModeThumbs;
+    else if (obj.isName("FullScreen"))
+      pageMode = pageModeFullScreen;
+    else if (obj.isName("UseOC"))
+      pageMode = pageModeOC;
+  }
+  obj.free();
+
+  pageLayout = pageLayoutNone;
+  if (catDict.dictLookup("PageLayout", &obj)->isName()) {
+    if (obj.isName("SinglePage"))
+      pageLayout = pageLayoutSinglePage;
+    if (obj.isName("OneColumn"))
+      pageLayout = pageLayoutOneColumn;
+    if (obj.isName("TwoColumnLeft"))
+      pageLayout = pageLayoutTwoColumnLeft;
+    if (obj.isName("TwoColumnRight"))
+      pageLayout = pageLayoutTwoColumnRight;
+    if (obj.isName("TwoPageLeft"))
+      pageLayout = pageLayoutTwoPageLeft;
+    if (obj.isName("TwoPageRight"))
+      pageLayout = pageLayoutTwoPageRight;
+  }
   obj.free();
 
   // read base URI
@@ -124,7 +165,6 @@ Catalog::Catalog(XRef *xrefA) {
  err1:
   catDict.free();
   dests.initNull();
-  nameTree.initNull();
   ok = gFalse;
 }
 
@@ -141,10 +181,11 @@ Catalog::~Catalog() {
     gfree(pageRefs);
   }
   dests.free();
-  nameTree.free();
+  destNameTree.free();
   if (baseURI) {
     delete baseURI;
   }
+  delete pageLabelInfo;
   metadata.free();
   structTreeRoot.free();
   outline.free();
@@ -272,8 +313,8 @@ LinkDest *Catalog::findDest(GooString *name) {
     else
       obj1.free();
   }
-  if (!found && nameTree.isDict()) {
-    if (!findDestInTree(&nameTree, name, &obj1)->isNull())
+  if (!found) {
+    if (destNameTree.lookup(name, &obj1))
       found = gTrue;
     else
       obj1.free();
@@ -303,62 +344,136 @@ LinkDest *Catalog::findDest(GooString *name) {
   return dest;
 }
 
-Object *Catalog::findDestInTree(Object *tree, GooString *name, Object *obj) {
-  Object names, name1;
-  Object kids, kid, limits, low, high;
-  GBool done, found;
-  int cmp, i;
+NameTree::NameTree(void)
+{
+  size = 0;
+  length = 0;
+  entries = NULL;
+}
+
+NameTree::Entry::Entry(Array *array, int index) {
+  if (!array->getString(index, &name) || !array->getNF(index + 1, &value))
+    error(-1, "Invalid page tree");
+}
+
+NameTree::Entry::~Entry() {
+  value.free();
+}
+
+void NameTree::addEntry(Entry *entry)
+{
+  if (length == size) {
+    if (length == 0) {
+      size = 8;
+    } else {
+      size *= 2;
+    }
+    entries = (Entry **) grealloc (entries, sizeof (Entry *) * size);
+  }
+
+  entries[length] = entry;
+  ++length;
+}
+
+void NameTree::init(XRef *xrefA, Object *tree) {
+  xref = xrefA;
+  parse(tree);
+}
+
+void NameTree::parse(Object *tree) {
+  Object names;
+  Object kids, kid;
+  int i;
+
+  if (!tree->isDict())
+    return;
 
   // leaf node
   if (tree->dictLookup("Names", &names)->isArray()) {
-    done = found = gFalse;
-    for (i = 0; !done && i < names.arrayGetLength(); i += 2) {
-      if (names.arrayGet(i, &name1)->isString()) {
-	cmp = name->cmp(name1.getString());
-	if (cmp == 0) {
-	  names.arrayGet(i+1, obj);
-	  found = gTrue;
-	  done = gTrue;
-	} else if (cmp < 0) {
-	  done = gTrue;
-	}
-      }
-      name1.free();
+    for (i = 0; i < names.arrayGetLength(); i += 2) {
+      NameTree::Entry *entry;
+
+      entry = new Entry(names.getArray(), i);
+      addEntry(entry);
     }
-    names.free();
-    if (!found)
-      obj->initNull();
-    return obj;
   }
-  names.free();
 
   // root or intermediate node
-  done = gFalse;
   if (tree->dictLookup("Kids", &kids)->isArray()) {
-    for (i = 0; !done && i < kids.arrayGetLength(); ++i) {
-      if (kids.arrayGet(i, &kid)->isDict()) {
-	if (kid.dictLookup("Limits", &limits)->isArray()) {
-	  if (limits.arrayGet(0, &low)->isString() &&
-	      name->cmp(low.getString()) >= 0) {
-	    if (limits.arrayGet(1, &high)->isString() &&
-		name->cmp(high.getString()) <= 0) {
-	      findDestInTree(&kid, name, obj);
-	      done = gTrue;
-	    }
-	    high.free();
-	  }
-	  low.free();
-	}
-	limits.free();
-      }
+    for (i = 0; i < kids.arrayGetLength(); ++i) {
+      if (kids.arrayGet(i, &kid)->isDict())
+	parse(&kid);
       kid.free();
     }
   }
   kids.free();
+}
 
-  // name was outside of ranges of all kids
-  if (!done)
+int NameTree::Entry::cmp(const void *voidKey, const void *voidEntry)
+{
+  GooString *key = (GooString *) voidKey;
+  Entry *entry = *(NameTree::Entry **) voidEntry;
+
+  return key->cmp(&entry->name);
+}
+
+GBool NameTree::lookup(GooString *name, Object *obj)
+{
+  Entry **entry;
+
+  entry = (Entry **) bsearch(name, entries,
+			     length, sizeof(Entry *), Entry::cmp);
+  if (entry != NULL) {
+    (*entry)->value.fetch(xref, obj);
+    return gTrue;
+  } else {
+    printf("failed to look up %s\n", name->getCString());
     obj->initNull();
+    return gFalse;
+  }
+}
+
+void NameTree::free()
+{
+  int i;
+
+  for (i = 0; i < length; i++)
+    delete entries[i];
+
+  gfree(entries);
+}
+
+GBool Catalog::labelToIndex(GooString *label, int *index)
+{
+  char *end;
+
+  if (pageLabelInfo != NULL) {
+    if (!pageLabelInfo->labelToIndex(label, index))
+      return gFalse;
+  } else {
+    *index = strtol(label->getCString(), &end, 10) - 1;
+    if (*end != '\0')
+      return gFalse;
+  }
+
+  if (*index < 0 || *index >= numPages)
+    return gFalse;
+
+  return gTrue;
+}
+
+GBool Catalog::indexToLabel(int index, GooString *label)
+{
+  char buffer[32];
+
+  if (index < 0 || index >= numPages)
+    return gFalse;
 
-  return obj;
+  if (pageLabelInfo != NULL) {
+    return pageLabelInfo->indexToLabel(index, label);
+  } else {
+    snprintf(buffer, sizeof (buffer), "%d", index + 1);
+    label->append(buffer);	      
+    return gTrue;
+  }
 }
