@@ -19,6 +19,37 @@ class Page;
 class PageAttrs;
 struct Ref;
 class LinkDest;
+class PageLabelInfo;
+
+//------------------------------------------------------------------------
+// NameTree
+//------------------------------------------------------------------------
+
+class NameTree {
+public:
+  NameTree();
+  void init(XRef *xref, Object *tree);
+  void parse(Object *tree);
+  GBool lookup(GooString *name, Object *obj);
+  void free();
+
+private:
+  struct Entry {
+    Entry(Array *array, int index);
+    ~Entry();
+    GooString name;
+    Object value;
+    void free();
+    static int cmp(const void *key, const void *entry);
+  };
+
+  void addEntry(Entry *entry);
+
+  XRef *xref;
+  Object *root;
+  Entry **entries;
+  int size, length;
+};
 
 //------------------------------------------------------------------------
 // Catalog
@@ -63,8 +94,33 @@ public:
   // NULL if <name> is not a destination.
   LinkDest *findDest(GooString *name);
 
+  // Convert between page indices and page labels.
+  GBool labelToIndex(GooString *label, int *index);
+  GBool indexToLabel(int index, GooString *label);
+
   Object *getOutline() { return &outline; }
 
+  enum PageMode {
+    pageModeNone,
+    pageModeOutlines,
+    pageModeThumbs,
+    pageModeFullScreen,
+    pageModeOC
+  };
+  enum PageLayout {
+    pageLayoutNone,
+    pageLayoutSinglePage,
+    pageLayoutOneColumn,
+    pageLayoutTwoColumnLeft,
+    pageLayoutTwoColumnRight,
+    pageLayoutTwoPageLeft,
+    pageLayoutTwoPageRight,
+  };
+
+  // Returns the page mode.
+  PageMode getPageMode() { return pageMode; }
+  PageLayout getPageLayout() { return pageLayout; }
+
 private:
 
   XRef *xref;			// the xref table for this PDF file
@@ -73,12 +129,15 @@ private:
   int numPages;			// number of pages
   int pagesSize;		// size of pages array
   Object dests;			// named destination dictionary
-  Object nameTree;		// name tree
+  NameTree destNameTree;	// name tree
   GooString *baseURI;		// base URI for URI-type links
   Object metadata;		// metadata stream
   Object structTreeRoot;	// structure tree root dictionary
   Object outline;		// outline dictionary
   GBool ok;			// true if catalog is valid
+  PageLabelInfo *pageLabelInfo; // info about page labels
+  PageMode pageMode;		// page mode
+  PageLayout pageLayout;	// page layout
 
   int readPageTree(Dict *pages, PageAttrs *attrs, int start);
   Object *findDestInTree(Object *tree, GooString *name, Object *obj);
